@@ -118,23 +118,26 @@ inline int read_matrix(Transform& t, char* line, size_t len) {
   return n;
 }
 
-inline ResidueId read_res_id(const char* seq_id, const char* name) {
-  ResidueId rid;
-  if (seq_id[4] != '\r' && seq_id[4] != '\n')
-    rid.seqid.icode = seq_id[4];
+inline SeqId read_seq_id(const char* str) {
+  SeqId seqid;
+  if (str[4] != '\r' && str[4] != '\n')
+    seqid.icode = str[4];
   // We support hybrid-36 extension, although it is never used in practice
   // as 9999 residues per chain are enough.
-  if (seq_id[0] < 'A') {
-    for (int i = 4; i != 0; --i, ++seq_id)
-      if (!is_space(*seq_id)) {
-        rid.seqid.num = read_int(seq_id, i);
+  if (str[0] < 'A') {
+    for (int i = 4; i != 0; --i, ++str)
+      if (!is_space(*str)) {
+        seqid.num = read_int(str, i);
         break;
       }
   } else {
-    rid.seqid.num = read_base36<4>(seq_id) - 466560 + 10000;
+    seqid.num = read_base36<4>(str) - 466560 + 10000;
   }
-  rid.name = read_string(name, 3);
-  return rid;
+  return seqid;
+}
+
+inline ResidueId read_res_id(const char* seq_id, const char* name) {
+  return {read_seq_id(seq_id), {}, read_string(name, 3)};
 }
 
 inline char read_altloc(char c) { return c == ' ' ? '\0' : c; }
@@ -277,7 +280,8 @@ inline bool same_str(const std::string& s, const char (&literal)[N]) {
 }
 
 template<typename Input>
-Structure read_pdb_from_line_input(Input&& infile, const std::string& source) {
+Structure read_pdb_from_line_input(Input&& infile, const std::string& source,
+                                   int max_line_length) {
   using namespace pdb_impl;
   int line_num = 0;
   auto wrong = [&line_num](const std::string& msg) {
@@ -292,9 +296,11 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source) {
   Chain *chain = nullptr;
   Residue *resi = nullptr;
   char line[122] = {0};
+  if (max_line_length <= 0 || max_line_length > 120)
+    max_line_length = 120;
   bool after_ter = false;
   Transform matrix;
-  while (size_t len = copy_line_from_stream(line, 121, infile)) {
+  while (size_t len = copy_line_from_stream(line, max_line_length+1, infile)) {
     ++line_num;
     if (is_record_type(line, "ATOM") || is_record_type(line, "HETATM")) {
       if (len < 66)
@@ -438,6 +444,33 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source) {
           ent.full_sequence.emplace_back(res_name);
       }
 
+    } else if (is_record_type(line, "DBREF")) { // DBREF or DBREF1 or DBREF2
+      std::string chain_name = read_string(line+11, 2);
+      Entity& ent = impl::find_or_add(st.entities, chain_name);
+      if (line[5] == ' ' || line[5] == '1')
+        ent.dbrefs.emplace_back();
+      else if (ent.dbrefs.empty()) // DBREF2 without DBREF1?
+        continue;
+      Entity::DbRef& dbref = ent.dbrefs.back();
+      if (line[5] == ' ' || line[5] == '1') {
+        dbref.seq_begin = read_seq_id(line+14);
+        dbref.seq_end = read_seq_id(line+20);
+        dbref.db_name = read_string(line+26, 6);
+        if (line[5] == ' ') {
+          dbref.accession_code = read_string(line+33, 8);
+          dbref.id_code = read_string(line+42, 12);
+          dbref.db_begin.num = read_int(line+55, 5);
+          dbref.db_begin.icode = line[60];
+          dbref.db_end.num = read_int(line+62, 5);
+          dbref.db_end.icode = line[67];
+        } else {  // line[5] == '1'
+          dbref.id_code = read_string(line+47, 20);
+        }
+      } else if (line[5] == '2') {
+        dbref.accession_code = read_string(line+18, 22);
+        dbref.db_begin.num = read_int(line+45, 10);
+        dbref.db_end.num = read_int(line+57, 10);
+      }
     } else if (is_record_type(line, "HEADER")) {
       if (len > 50)
         st.info["_struct_keywords.pdbx_keywords"] = rtrim_str(std::string(line+10, 40));
@@ -517,9 +550,9 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source) {
       if (len < 40)
         continue;
       Helix helix;
-      helix.start.chain_name = read_string(line+19, 2);
+      helix.start.chain_name = read_string(line+18, 2);
       helix.start.res_id = read_res_id(line+21, line+15);
-      helix.end.chain_name = read_string(line+31, 2);
+      helix.end.chain_name = read_string(line+30, 2);
       helix.end.res_id = read_res_id(line+33, line+27);
       helix.set_helix_class_as_int(read_int(line+38, 2));
       if (len > 72)
@@ -533,9 +566,9 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source) {
       Sheet& sheet = impl::find_or_add(st.sheets, sheet_id);
       sheet.strands.emplace_back();
       Sheet::Strand& strand = sheet.strands.back();
-      strand.start.chain_name = read_string(line+21, 2);
+      strand.start.chain_name = read_string(line+20, 2);
       strand.start.res_id = read_res_id(line+22, line+17);
-      strand.end.chain_name = read_string(line+32, 2);
+      strand.end.chain_name = read_string(line+31, 2);
       strand.end.res_id = read_res_id(line+33, line+28);
       strand.sense = read_int(line+38, 2);
       if (len > 67) {
@@ -579,15 +612,15 @@ Structure read_pdb_from_line_input(Input&& infile, const std::string& source) {
 
 }  // namespace pdb_impl
 
-inline Structure read_pdb_file(const std::string& path) {
+inline Structure read_pdb_file(const std::string& path, int linelen=0) {
   auto f = file_open(path.c_str(), "rb");
-  return pdb_impl::read_pdb_from_line_input(FileStream{f.get()}, path);
+  return pdb_impl::read_pdb_from_line_input(FileStream{f.get()}, path, linelen);
 }
 
 inline Structure read_pdb_from_memory(const char* data, size_t size,
-                                      const std::string& name) {
+                                      const std::string& name, int linelen=0) {
   return pdb_impl::read_pdb_from_line_input(MemoryStream{data, data + size},
-                                            name);
+                                            name, linelen);
 }
 
 inline Structure read_pdb_string(const std::string& str,
@@ -597,13 +630,14 @@ inline Structure read_pdb_string(const std::string& str,
 
 // A function for transparent reading of stdin and/or gzipped files.
 template<typename T>
-inline Structure read_pdb(T&& input) {
+inline Structure read_pdb(T&& input, int linelen=0) {
   if (input.is_stdin())
-    return pdb_impl::read_pdb_from_line_input(FileStream{stdin}, "stdin");
+    return pdb_impl::read_pdb_from_line_input(FileStream{stdin},
+                                              "stdin", linelen);
   if (input.is_compressed())
     return pdb_impl::read_pdb_from_line_input(input.get_uncompressing_stream(),
-                                              input.path());
-  return read_pdb_file(input.path());
+                                              input.path(), linelen);
+  return read_pdb_file(input.path(), linelen);
 }
 
 } // namespace gemmi
