@@ -19,7 +19,8 @@ void add_mol(py::module& m); // mol.cpp
 void add_cif(py::module& cif); // cif.cpp
 void add_read_structure(py::module& m); // read.cpp
 void add_cif_read(py::module& cif); // read.cpp
-void add_monlib(py::module& cif); // monlib.cpp
+void add_monlib(py::module& m); // monlib.cpp
+void add_alignment(py::module& m); // monlib.cpp
 
 void add_misc(py::module& m) {
   using gemmi::Element;
@@ -123,6 +124,7 @@ PYBIND11_MODULE(gemmi, mg) {
   add_misc(mg);
   add_mol(mg);
   add_monlib(mg);
+  add_alignment(mg);
   add_read_structure(mg);
   py::module cif = mg.def_submodule("cif", "CIF file format");
   add_cif(cif);
