@@ -6,6 +6,8 @@
 #include "gemmi/model.hpp"
 #include "gemmi/calculate.hpp"
 #include "gemmi/polyheur.hpp"
+#include "gemmi/assembly.hpp"
+#include "gemmi/labelseq.hpp"   // for setup_for_mmcif
 #include "gemmi/to_pdb.hpp"
 #include "gemmi/to_mmcif.hpp"
 #include "gemmi/tostr.hpp"
@@ -22,6 +24,7 @@ using namespace gemmi;
 PYBIND11_MAKE_OPAQUE(std::vector<Connection>)
 PYBIND11_MAKE_OPAQUE(std::vector<NcsOp>)
 PYBIND11_MAKE_OPAQUE(std::vector<Entity>)
+PYBIND11_MAKE_OPAQUE(std::vector<Assembly>)
 using info_map_type = std::map<std::string, std::string>;
 PYBIND11_MAKE_OPAQUE(info_map_type)
 
@@ -53,7 +56,7 @@ C& add_child(P& parent, C child, int pos) {
   return add_item(parent.children(), std::move(child), pos);
 }
 
-template<typename T> int normalize_index(int index, T& container) {
+template<typename T> int normalize_index(int index, const T& container) {
   if (index < 0)
     index += (int) container.size();
   if ((size_t) index >= container.size())
@@ -61,11 +64,36 @@ template<typename T> int normalize_index(int index, T& container) {
   return index;
 }
 
+template<typename P, typename C> C& get_child(P& parent, int index) {
+  auto& children = parent.children();
+  return children[normalize_index(index, children)];
+}
+
 template<typename P> void remove_child(P& parent, int index) {
   auto& children = parent.children();
   children.erase(children.begin() + normalize_index(index, children));
 }
 
+template<typename P> void remove_children(P& parent, py::slice slice) {
+  ssize_t start, stop, step, slength;
+  auto& children = parent.children();
+  if (!slice.compute(children.size(), &start, &stop, &step, &slength))
+    throw py::error_already_set();
+  for (int i = 0; i < slength; ++i) {
+    ssize_t idx = start + (step > 0 ? slength - 1 - i : i) * step;
+    children.erase(children.begin() + idx);
+  }
+}
+
+static std::vector<std::string>
+expand_protein_one_letter_string(const std::string& s) {
+  std::vector<std::string> r;
+  r.reserve(s.size());
+  for (char c : s)
+    r.push_back(expand_protein_one_letter(c));
+  return r;
+}
+
 void add_mol(py::module& m) {
   py::class_<ResidueInfo>(m, "ResidueInfo")
     .def_readonly("one_letter_code", &ResidueInfo::one_letter_code)
@@ -79,6 +107,12 @@ void add_mol(py::module& m) {
 
   m.def("find_tabulated_residue", &find_tabulated_residue, py::arg("name"),
         "Find chemical component information in the internal table.");
+  m.def("expand_protein_one_letter", &expand_protein_one_letter);
+  m.def("expand_protein_one_letter_string", &expand_protein_one_letter_string);
+  m.def("one_letter_code",
+        (std::string (*)(const std::vector<std::string>&)) &one_letter_code);
+  m.def("one_letter_code",
+        [](const ResidueSpan& span) { return one_letter_code(span); });
 
   py::enum_<CoorFormat>(m, "CoorFormat")
     .value("Unknown", CoorFormat::Unknown)
@@ -107,6 +141,12 @@ void add_mol(py::module& m) {
     .value("Other", PolymerType::Other)
     .value("Unknown", PolymerType::Unknown);
 
+  py::enum_<HowToNameCopiedChains>(m, "HowToNameCopiedChains")
+    .value("Short", HowToNameCopiedChains::Short)
+    .value("AddNumber", HowToNameCopiedChains::AddNumber)
+    .value("Dup", HowToNameCopiedChains::Dup)
+    ;
+
   py::class_<Entity>(m, "Entity")
     .def(py::init<std::string>())
     .def_readwrite("name", &Entity::name)
@@ -129,6 +169,15 @@ void add_mol(py::module& m) {
                      (self.given ? " (" : " (not "), "given)>");
     });
 
+  py::class_<Assembly>(m, "Assembly")
+    .def_readonly("name", &Assembly::name)
+    .def_readonly("oligomeric_details", &Assembly::oligomeric_details)
+    .def("make_assembly", [](const Assembly& self, const Model& model,
+                             HowToNameCopiedChains how) {
+        return make_assembly(self, model, how, nullptr);
+    })
+    ;
+
   py::enum_<Connection::Type>(m, "ConnectionType")
     .value("Covale", Connection::Type::Covale)
     .value("Disulf", Connection::Type::Disulf)
@@ -153,6 +202,7 @@ void add_mol(py::module& m) {
   py::bind_vector<std::vector<Connection>>(m, "ConnectionList");
   py::bind_vector<std::vector<NcsOp>>(m, "NcsOpList");
   py::bind_vector<std::vector<Entity>>(m, "EntityList");
+  py::bind_vector<std::vector<Assembly>>(m, "AssemblyList");
   py::bind_map<info_map_type>(m, "InfoMap");
 
   py::class_<Structure>(m, "Structure")
@@ -164,6 +214,7 @@ void add_mol(py::module& m) {
     .def_readwrite("resolution", &Structure::resolution)
     .def_readwrite("entities", &Structure::entities)
     .def_readwrite("connections", &Structure::connections)
+    .def_readwrite("assemblies", &Structure::assemblies)
     .def_readwrite("info", &Structure::info)
     .def("get_entity",
          (Entity* (Structure::*)(const std::string&)) &Structure::get_entity,
@@ -175,12 +226,13 @@ void add_mol(py::module& m) {
     .def("__iter__", [](const Structure& st) {
         return py::make_iterator(st.models);
     }, py::keep_alive<0, 1>())
-    .def("__getitem__", [](Structure& st, int index) -> Model& {
-        return st.models.at(index >= 0 ? index : index + st.models.size());
-    }, py::arg("index"), py::return_value_policy::reference_internal)
+    .def("__getitem__", &get_child<Structure, Model>, py::arg("index"),
+         py::return_value_policy::reference_internal)
     .def("__getitem__", [](Structure& st, const std::string& name) -> Model& {
         return *impl::find_iter(st.models, name);
     }, py::arg("name"), py::return_value_policy::reference_internal)
+    .def("__delitem__", &remove_child<Structure>, py::arg("index"))
+    .def("__delitem__", &remove_children<Structure>)
     .def("__delitem__", &Structure::remove_model, py::arg("name"))
     .def("find_connection", &Structure::find_connection,
          py::arg("partner1"), py::arg("partner2"),
@@ -234,6 +286,9 @@ void add_mol(py::module& m) {
     .def("ensure_entities", &ensure_entities)
     .def("deduplicate_entities", &deduplicate_entities)
     .def("setup_entities", &setup_entities)
+    .def("setup_for_mmcif", &setup_for_mmcif)
+    .def("remove_alternative_conformations",
+         remove_alternative_conformations<Structure>)
     .def("remove_hydrogens", remove_hydrogens<Structure>)
     .def("remove_waters", remove_waters<Structure>)
     .def("remove_ligands_and_waters",
@@ -276,12 +331,11 @@ void add_mol(py::module& m) {
     .def("__iter__", [](const Model& self) {
         return py::make_iterator(self.chains);
     }, py::keep_alive<0, 1>())
+    .def("__getitem__", &get_child<Model, Chain>, py::arg("index"),
+         py::return_value_policy::reference_internal)
     .def("__getitem__", [](Model& self, const std::string& name) -> Chain& {
         return *impl::find_iter(self.chains, name);
     }, py::arg("name"), py::return_value_policy::reference_internal)
-    .def("__getitem__", [](Model& self, int index) -> Chain& {
-        return self.chains.at(index >= 0 ? index : index + self.chains.size());
-    }, py::arg("index"), py::return_value_policy::reference_internal)
     .def("get_subchain",
          (ResidueSpan (Model::*)(const std::string&)) &Model::get_subchain,
          py::arg("name"), py::return_value_policy::reference_internal)
@@ -295,7 +349,8 @@ void add_mol(py::module& m) {
          py::return_value_policy::reference_internal)
     .def("get_all_residue_names", &Model::get_all_residue_names)
     .def("find_cra", (CRA (Model::*)(const AtomAddress&)) &Model::find_cra)
-    .def("find_chain", &Model::find_chain,
+    .def("find_chain",
+         (Chain* (Model::*)(const std::string&))&Model::find_chain,
          py::arg("name"), py::return_value_policy::reference_internal)
     .def("find_last_chain", &Model::find_last_chain,
          py::arg("name"), py::return_value_policy::reference_internal)
@@ -309,8 +364,11 @@ void add_mol(py::module& m) {
      }, py::arg("name"), py::return_value_policy::reference_internal)
     .def("remove_chain", &Model::remove_chain, py::arg("name"))
     .def("__delitem__", &Model::remove_chain, py::arg("name"))
+    .def("__delitem__", remove_child<Model>, py::arg("index"))
+    .def("__delitem__", remove_children<Model>)
     .def("count_atom_sites", &count_atom_sites<Model>)
     .def("count_occupancies", &count_occupancies<Model>)
+    .def("calculate_mass", &calculate_mass<Model>)
     .def("calculate_center_of_mass", [](const Model& self) {
         return calculate_center_of_mass(self).get();
     })
@@ -349,19 +407,19 @@ void add_mol(py::module& m) {
     .def("__getitem__", [](Chain& ch, const std::string& seqid) {
         return ch.find_residue_group(SeqId(seqid));
     }, py::arg("pdb_seqid"), py::keep_alive<0, 1>())
-    .def("__getitem__", [](Chain& ch, int index) -> Residue& {
-        return ch.residues.at(index >= 0 ? index : index + ch.residues.size());
-    }, py::arg("index"), py::return_value_policy::reference_internal)
+    .def("__getitem__", &get_child<Chain, Residue>, py::arg("index"),
+         py::return_value_policy::reference_internal)
     .def("__getitem__", [](Chain &ch, py::slice slice) -> py::list {
-        size_t start, stop, step, slength;
+        ssize_t start, stop, step, slength;
         if (!slice.compute(ch.residues.size(), &start, &stop, &step, &slength))
           throw py::error_already_set();
         py::list l;
-        for (size_t i = 0; i < slength; ++i)
+        for (ssize_t i = 0; i < slength; ++i)
           l.append(py::cast(&ch.residues[start + i * step]));
         return l;
     }, py::return_value_policy::reference_internal)
     .def("__delitem__", remove_child<Chain>, py::arg("index"))
+    .def("__delitem__", remove_children<Chain>)
     .def("add_residue", add_child<Chain, Residue>,
          py::arg("residue"), py::arg("pos")=-1,
          py::return_value_policy::reference_internal)
@@ -381,6 +439,7 @@ void add_mol(py::module& m) {
          py::arg("new_residues"), py::arg("min_sep")=0)
     .def("count_atom_sites", &count_atom_sites<Chain>)
     .def("count_occupancies", &count_occupancies<Chain>)
+    .def("calculate_mass", &calculate_mass<Chain>)
     .def("trim_to_alanine", (void (*)(Chain&)) &trim_to_alanine)
     .def("first_conformer",
          (UniqProxy<Residue> (Chain::*)()) &Chain::first_conformer,
@@ -396,7 +455,7 @@ void add_mol(py::module& m) {
          py::keep_alive<0, 1>())
     .def("__bool__", [](const ResidueSpan &g) -> bool { return !g.empty(); })
     .def("__getitem__", [](ResidueSpan& g, int index) -> Residue& {
-        return g.at(index >= 0 ? index : index + g.size());
+        return g[normalize_index(index, g)];
     }, py::arg("index"), py::return_value_policy::reference_internal)
     .def("__getitem__", [](ResidueSpan& self, const std::string& seqid) {
         return self.find_residue_group(SeqId(seqid));
@@ -414,6 +473,8 @@ void add_mol(py::module& m) {
     .def("residue_groups", &ResidueSpan::residue_groups, py::keep_alive<0, 1>())
     .def("length", &ResidueSpan::length)
     .def("subchain_id", &ResidueSpan::subchain_id)
+    .def("label_seq_id_to_auth", &ResidueSpan::label_seq_id_to_auth)
+    .def("auth_seq_id_to_label", &ResidueSpan::auth_seq_id_to_label)
     .def("check_polymer_type", [](const ResidueSpan& span) {
         return check_polymer_type(span);
     })
@@ -435,7 +496,7 @@ void add_mol(py::module& m) {
   py::class_<ResidueGroup, ResidueSpan>(m, "ResidueGroup")
     // need to duplicate it so it is visible
     .def("__getitem__", [](ResidueGroup& g, int index) -> Residue& {
-        return g.at(index >= 0 ? index : index + g.size());
+        return g[normalize_index(index, g)];
     }, py::arg("index"), py::return_value_policy::reference_internal)
     .def("__getitem__", &ResidueGroup::by_resname,
          py::arg("name"), py::return_value_policy::reference_internal)
@@ -452,7 +513,7 @@ void add_mol(py::module& m) {
          py::keep_alive<0, 1>())
     .def("__bool__", [](const ResidueSpan &g) -> bool { return !g.empty(); })
     .def("__getitem__", [](AtomGroup& g, int index) -> Atom& {
-        return g.at(index >= 0 ? index : index + g.size());
+        return g[normalize_index(index, g)];
     }, py::arg("index"), py::return_value_policy::reference_internal)
     .def("__getitem__", &AtomGroup::by_altloc,
          py::arg("altloc"), py::return_value_policy::reference_internal)
@@ -495,12 +556,12 @@ void add_mol(py::module& m) {
     .def("__iter__", [](const Residue& res) {
         return py::make_iterator(res.atoms);
     }, py::keep_alive<0, 1>())
-    .def("__getitem__", [](Residue& self, int index) -> Atom& {
-        return self.atoms.at(index >= 0 ? index : index + self.atoms.size());
-    }, py::arg("index"), py::return_value_policy::reference_internal)
+    .def("__getitem__", &get_child<Residue, Atom>, py::arg("index"),
+         py::return_value_policy::reference_internal)
     .def("__getitem__", &Residue::get,
          py::arg("name"), py::return_value_policy::reference_internal)
     .def("__delitem__", remove_child<Residue>, py::arg("index"))
+    .def("__delitem__", remove_children<Residue>)
     .def("find_atom", [](Residue& self, const std::string& name, char altloc) {
            return self.find_atom(name, altloc);
          },
